@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2006-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -62,7 +62,7 @@ elif os.name == 'nt':
 	import win32net
 	import win32security
 
-__version__ = '4.0.6.39'
+__version__ = '4.0.7.4'
 
 logger = Logger()
 
@@ -405,8 +405,7 @@ class BackendDispatcher(Backend):
 
 		try:
 			self._dispatchConfig = BackendDispatchConfigFile(self._dispatchConfigFile).parse()
-			logger.debug(u"Read dispatch config from file '%s':" % self._dispatchConfigFile)
-			logger.debug(objectToBeautifiedText(self._dispatchConfig))
+			logger.debug(u"Read dispatch config from file {0!r}: {1!r}", self._dispatchConfigFile, self._dispatchConfig)
 		except Exception as e:
 			raise BackendConfigurationError(u"Failed to load dispatch config file '%s': %s" % (self._dispatchConfigFile, e))
 
@@ -456,7 +455,7 @@ class BackendDispatcher(Backend):
 				logger.debug2(u"Found public %s method '%s'" % (Class.__name__, methodName))
 
 				if hasattr(self, methodName):
-					logger.debug(u"{0}: skipping already present method {1}".format(self.__class__.__name__, methodName))
+					logger.debug(u"{0}: skipping already present method {1}", self.__class__.__name__, methodName)
 					continue
 
 				methodBackends = []
@@ -466,10 +465,10 @@ class BackendDispatcher(Backend):
 
 					for backend in forceList(backends):
 						if backend not in self._backends:
-							logger.debug(u"Ignoring backend {0!r}: backend not available".format(backend))
+							logger.debug(u"Ignoring backend {0!r}: backend not available", backend)
 							continue
 						methodBackends.append(backend)
-					logger.debug(u"{0!r} matches method {1!r}, dispatching to backends: {2}".format(regex, methodName, u', '.join(methodBackends)))
+					logger.debug(u"{0!r} matches method {1!r}, dispatching to backends: {2}", regex, methodName, u', '.join(methodBackends))
 					break
 
 				if not methodBackends:
@@ -481,7 +480,7 @@ class BackendDispatcher(Backend):
 				setattr(self, methodName, new.instancemethod(eval(methodName), self, self.__class__))
 
 	def _dispatchMethod(self, methodBackends, methodName, **kwargs):
-		logger.debug(u"Dispatching method {0!r} to backends: {1}".format(methodName, methodBackends))
+		logger.debug(u"Dispatching method {0!r} to backends: {1}", methodName, methodBackends)
 		result = None
 
 		for methodBackend in methodBackends:
@@ -500,7 +499,7 @@ class BackendDispatcher(Backend):
 			elif res is not None:
 				result = res
 
-		logger.debug2(u"Finished dispatching method {0!r}".format(methodName))
+		logger.debug2(u"Finished dispatching method {0!r}", methodName)
 		return result
 
 	def backend_setOptions(self, options):
@@ -550,7 +549,7 @@ class BackendExtender(ExtendedBackend):
 			for methodName, functionRef in inspect.getmembers(self._extensionClass, inspect.ismethod):
 				if methodName.startswith('_'):
 					continue
-				logger.debug2(u"Extending {0} with instancemethod: {1!r}".format(self._backend.__class__.__name__, methodName))
+				logger.debug2(u"Extending {0} with instancemethod: {1!r}", self._backend.__class__.__name__, methodName)
 				new_function = new.function(functionRef.func_code, functionRef.func_globals, functionRef.func_code.co_name)
 				new_method = new.instancemethod(new_function, self, self.__class__)
 				setattr(self, methodName, new_method)
@@ -628,8 +627,6 @@ class BackendAccessControl(object):
 				if value is not None:
 					self._forceGroups = forceUnicodeList(value)
 
-		if not self._acl:
-			self._acl = [['.*', [{'type': u'sys_group', 'ids': [u'opsiadmin'], 'denyAttributes': [], 'allowAttributes': []}]]]
 		if not self._username:
 			raise BackendAuthenticationError(u"No username specified")
 		if not self._password:
@@ -639,11 +636,6 @@ class BackendAccessControl(object):
 		if isinstance(self._backend, BackendAccessControl):
 			raise BackendConfigurationError(u"Cannot use BackendAccessControl instance as backend")
 
-		# TODO: forceACL
-		# for i in range(len(self._acl)):
-		# 	self._acl[i][0] = re.compile(self._acl[i][0])
-		# 	self._acl[i][1] = forceUnicodeList(self._acl[i][1])
-
 		try:
 			if re.search('^[^\.]+\.[^\.]+\.\S+$', self._username):
 				# Username starts with something like hostname.domain.tld:
@@ -651,23 +643,27 @@ class BackendAccessControl(object):
 				logger.debug(u"Trying to authenticate by opsiHostKey...")
 				self._username = self._username.lower()
 
-				if not hasattr(self._context, 'host_getObjects'):
+				try:
+					host = self._context.host_getObjects(id=self._username)
+				except AttributeError as aerr:
+					logger.debug(u"{0!r}", aerr)
 					raise Exception(u"Passed backend has no method 'host_getObjects', cannot authenticate host '%s'" % self._username)
 
-				host = self._context.host_getObjects(id=self._username)
-				if not host:
+				try:
+					self._host = host[0]
+				except IndexError as ierr:
+					logger.debug(u"{0!r}", ierr)
 					raise Exception(u"Host '%s' not found in backend %s" % (self._username, self._context))
-				self._host = host[0]
 
 				if not self._host.opsiHostKey:
 					raise Exception(u"OpsiHostKey not found for host '%s'" % self._username)
 
-				logger.confidential(u"Client '%s', key sent '%s', key stored '%s'" % (self._username, self._password, self._host.opsiHostKey))
+				logger.confidential(u"Client {0!r}, key sent {1!r}, key stored {2!r}", self._username, self._password, self._host.opsiHostKey)
 
 				if self._password != self._host.opsiHostKey:
 					raise BackendAuthenticationError(u"OpsiHostKey authentication failed for host '%s': wrong key" % self._host.id)
 
-				logger.info(u"OpsiHostKey authentication successful for host '%s'" % self._host.id)
+				logger.info(u"OpsiHostKey authentication successful for host {0!r}", self._host.id)
 			else:
 				# System user trying to log in with username and password
 				logger.debug(u"Trying to authenticate by operating system...")
@@ -675,13 +671,20 @@ class BackendAccessControl(object):
 				# Authentication did not throw exception => authentication successful
 				logger.info(u"Operating system authentication successful for user '%s', groups '%s'" % (self._username, ','.join(self._userGroups)))
 		except Exception as e:
-			raise BackendAuthenticationError(u"%s" % e)
+			raise BackendAuthenticationError(forceUnicode(e))
 
 		self._createInstanceMethods()
 		if self._aclFile:
 			self.__loadACLFile()
 		self._authenticated = True
 
+		if not self._acl:
+			self._acl = [['.*', [{'type': u'sys_group', 'ids': [u'opsiadmin'], 'denyAttributes': [], 'allowAttributes': []}]]]
+
+		# Pre-compiling regex patterns for speedup.
+		for i, (pattern, acl) in enumerate(self._acl):
+			self._acl[i] = (re.compile(pattern), acl)
+
 	def accessControl_authenticated(self):
 		return self._authenticated
 
@@ -701,8 +704,7 @@ class BackendAccessControl(object):
 			if not os.path.exists(self._aclFile):
 				raise Exception(u"Acl file '%s' not found" % self._aclFile)
 			self._acl = BackendACLFile(self._aclFile).parse()
-			logger.debug(u"Read acl from file '%s':" % self._aclFile)
-			logger.debug(objectToBeautifiedText(self._acl))
+			logger.debug(u"Read acl from file {0!r}: {1!r}", self._aclFile, self._acl)
 		except Exception as e:
 			logger.logException(e)
 			raise BackendConfigurationError(u"Failed to load acl file '%s': %s" % (self._aclFile, e))
@@ -769,7 +771,7 @@ class BackendAccessControl(object):
 								(username, domain, type) = win32security.LookupAccountSid(None, sid)
 								if username.lower() == self._username.lower():
 									self._userGroups.add(groupname)
-									logger.debug(u"User '%s' is member of group '%s'" % (self._username, groupname))
+									logger.debug(u"User {0!r} is member of group {1!r}", self._username, groupname)
 							if uresume == 0:
 								break
 						if gresume == 0:
@@ -785,7 +787,7 @@ class BackendAccessControl(object):
 
 		:raises BackendAuthenticationError: If authentication fails.
 		'''
-		logger.confidential(u"Trying to authenticate user '%s' with password '%s' by PAM" % (self._username, self._password))
+		logger.confidential(u"Trying to authenticate user {0!r} with password {1!r} by PAM", self._username, self._password)
 
 		class AuthConv:
 			''' Handle PAM conversation '''
@@ -796,7 +798,7 @@ class BackendAccessControl(object):
 			def __call__(self, auth, query_list, userData=None):
 				response = []
 				for (query, qtype) in query_list:
-					logger.debug(u"PAM conversation: query '%s', type '%s'" % (query, qtype))
+					logger.debug(u"PAM conversation: query {0!r}, type {1!r}", query, qtype)
 					if qtype == PAM.PAM_PROMPT_ECHO_ON:
 						response.append((self.user, 0))
 					elif qtype == PAM.PAM_PROMPT_ECHO_OFF:
@@ -831,11 +833,11 @@ class BackendAccessControl(object):
 				logger.info(u"Forced groups for user '%s': %s" % (self._username, self._userGroups))
 			else:
 				primaryGroup = forceUnicode(grp.getgrgid(pwd.getpwnam(self._username)[3])[0])
-				logger.debug(u"Primary group of user '{0}' is '{1}'".format(self._username, primaryGroup))
+				logger.debug(u"Primary group of user {0!r} is {1!r}", self._username, primaryGroup)
 
 				self._userGroups = set(forceUnicode(group[0]) for group in grp.getgrall() if self._username in group[3])
 				self._userGroups.add(primaryGroup)
-				logger.debug(u"User '{0}' is member of groups: {1}".format(self._username, self._userGroups))
+				logger.debug(u"User {0!r} is member of groups: {1}", self._username, self._userGroups)
 		except Exception as e:
 			raise BackendAuthenticationError(u"PAM authentication failed for user '%s': %s" % (self._username, e))
 
@@ -886,12 +888,14 @@ class BackendAccessControl(object):
 		granted = False
 		newKwargs = {}
 		acls = []
-		logger.debug(u"Access control for method '%s' params %s" % (methodName, kwargs))
-		for (regex, acl) in self._acl:
-			logger.debug2(u"Testing acl %s: %s for method '%s'" % (regex, acl, methodName))
-			if not re.search(regex, methodName):
+		logger.debug(u"Access control for method {0!r} with params {1!r}", methodName, kwargs)
+		for regex, acl in self._acl:
+			logger.debug2(u"Testing if ACL pattern {0!r} matches method {1!r}", regex.pattern, methodName)
+			if not regex.search(methodName):
+				logger.debug2(u"No match -> skipping.")
 				continue
-			logger.debug(u"Found matching acl %s for method '%s'" % (acl, methodName))
+
+			logger.debug(u"Found matching acl for method {1!r}: {0}", acl, methodName)
 			for entry in acl:
 				aclType = entry.get('type')
 				ids = entry.get('ids', [])
@@ -909,7 +913,7 @@ class BackendAccessControl(object):
 				elif aclType == 'self':
 					newGranted = 'partial_object'
 				else:
-					logger.error(u"Unhandled acl entry type: %s" % aclType)
+					logger.error(u"Unhandled acl entry type: {0}", aclType)
 					continue
 
 				if newGranted is False:
@@ -926,16 +930,15 @@ class BackendAccessControl(object):
 					break
 			break
 
-		logger.debug("Method {0!r} using acls: {1}".format(methodName, acls))
+		logger.debug("Method {0!r} using acls: {1}", methodName, acls)
 		if granted is True:
-			logger.debug(u"Full access to method '%s' granted to user '%s' by acl %s" % (methodName, self._username, acls[0]))
+			logger.debug(u"Full access to method {0!r} granted to user {1!r} by acl {2!r}", methodName, self._username, acls[0])
 			newKwargs = kwargs
 		elif granted is False:
 			raise BackendPermissionDeniedError(u"Access to method '%s' denied for user '%s'" % (methodName, self._username))
 		else:
-			logger.debug(u"Partial access to method '%s' granted to user '%s' by acls %s" % (methodName, self._username, acls))
+			logger.debug(u"Partial access to method {0!r} granted to user {1!r} by acls {2!r}", methodName, self._username, acls)
 			try:
-
 				newKwargs = self._filterParams(kwargs, acls)
 				if not newKwargs:
 					raise BackendPermissionDeniedError(u"No allowed param supplied")
@@ -943,7 +946,7 @@ class BackendAccessControl(object):
 				logger.logException(e, LOG_INFO)
 				raise BackendPermissionDeniedError(u"Access to method '%s' denied for user '%s': %s" % (methodName, self._username, e))
 
-		logger.debug("newKwargs: %s" % newKwargs)
+		logger.debug2("newKwargs: {0}", newKwargs)
 
 		meth = getattr(self._backend, methodName)
 		result = meth(**newKwargs)
@@ -951,12 +954,10 @@ class BackendAccessControl(object):
 		if granted is True:
 			return result
 
-		# Filter result
 		return self._filterResult(result, acls)
 
 	def _filterParams(self, params, acls):
-		params = dict(params)
-		logger.debug(u"Filtering params: %s" % params)
+		logger.debug(u"Filtering params: {0}", params)
 		for (key, value) in params.items():
 			valueList = forceList(value)
 			if len(valueList) == 0:
