@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2010-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -51,7 +51,7 @@ from OPSI.Util import fromJson, toJson
 from OPSI.Util.Thread import KillableThread
 from OPSI.Util.HTTP import closingConnection, non_blocking_connect_https
 
-__version__ = '4.0.6.39'
+__version__ = '4.0.7.2'
 
 logger = Logger()
 
@@ -97,7 +97,7 @@ class RpcThread(KillableThread):
 				connection.putheader('content-type', 'application/json')
 				connection.putheader('content-length', str(len(query)))
 				auth = u'{0}:{1}'.format(self.username, self.password)
-				connection.putheader('Authorization', 'Basic ' + base64.encodestring(auth.encode('latin-1')).strip())
+				connection.putheader('Authorization', 'Basic ' + base64.b64encode(auth.encode('latin-1')))
 				connection.endheaders()
 				connection.send(query)
 
@@ -196,8 +196,8 @@ class HostControlBackend(ExtendedBackend):
 		if self._resolveHostAddress:
 			try:
 				address = socket.gethostbyname(host.id)
-			except socket.error:
-				raise Exception(u"Failed to resolve ip address for host '%s'" % host.id)
+			except socket.error as lookupError:
+				logger.debug2("Failed to lookup ip address for {0}: {1!r}", host.id, lookupError)
 		if not address:
 			address = host.ipAddress
 		if not address and not self._resolveHostAddress:
