@@ -3,7 +3,7 @@
 
 # This module is part of the desktop management solution opsi
 # (open pc server integration) http://www.opsi.org
-# Copyright (C) 2013-2015 uib GmbH <info@uib.de>
+# Copyright (C) 2013-2016 uib GmbH <info@uib.de>
 # All rights reserved.
 
 # This program is free software: you can redistribute it and/or modify
@@ -28,8 +28,6 @@ MySQL-Backend
 :license: GNU Affero GPL version 3
 """
 
-__version__ = '4.0.6.1'
-
 import base64
 import warnings
 import time
@@ -49,6 +47,8 @@ from OPSI.Backend.Backend import ConfigDataBackend
 from OPSI.Backend.SQL import (onlyAllowSelect, SQL, SQLBackend,
 	SQLBackendObjectModificationTracker)
 
+__version__ = '4.0.6.29'
+
 logger = Logger()
 
 
@@ -61,13 +61,15 @@ class ConnectionPool(object):
 
 		# Check whether we already have an instance
 		if ConnectionPool.__instance is None:
-			logger.info(u"Creating ConnectionPool instance")
+			logger.debug(u"Creating ConnectionPool instance")
 			# Create and remember instance
 			poolArgs = {}
 			for key in ('pool_size', 'max_overflow', 'timeout'):
-				if key in kwargs.keys():
+				try:
 					poolArgs[key] = kwargs[key]
 					del kwargs[key]
+				except KeyError:
+					pass
 
 			def creator():
 				return MySQLdb.connect(**kwargs)
@@ -80,7 +82,7 @@ class ConnectionPool(object):
 		self.__dict__['_ConnectionPool__instance'] = ConnectionPool.__instance
 
 	def destroy(self):
-		logger.notice(u"Destroying ConnectionPool instance")
+		logger.debug(u"Destroying ConnectionPool instance")
 		ConnectionPool.__instance = None
 
 	def __getattr__(self, attr):
@@ -137,31 +139,46 @@ class MySQL(SQL):
 		self._createConnectionPool()
 		logger.debug(u'MySQL created: %s' % self)
 
+	def __repr__(self):
+		return u'<{0}(address={1!r})>'.format(self.__class__.__name__, self._address)
+
 	def _createConnectionPool(self):
 		logger.debug2(u"Creating connection pool")
-		self._transactionLock.acquire(0)
+		self._transactionLock.acquire(False)
 		try:
-			try:
-				if self._pool:
-					self._pool.destroy()
-				conv = dict(conversions)
-				conv[FIELD_TYPE.DATETIME] = str
-				conv[FIELD_TYPE.TIMESTAMP] = str
-				self._pool = ConnectionPool(
-						host=self._address,
-						user=self._username,
-						passwd=self._password,
-						db=self._database,
-						use_unicode=True,
-						charset=self._databaseCharset,
-						pool_size=self._connectionPoolSize,
-						max_overflow=self._connectionPoolMaxOverflow,
-						timeout=self._connectionPoolTimeout,
-						conv=conv
-				)
-			except Exception as error:
-				logger.logException(error)
-				raise BackendIOError(u"Failed to connect to database '%s' address '%s': %s" % (self._database, self._address, error))
+			for tryNumber in (1, 2):
+				try:
+					if self._pool:
+						self._pool.destroy()
+					conv = dict(conversions)
+					conv[FIELD_TYPE.DATETIME] = str
+					conv[FIELD_TYPE.TIMESTAMP] = str
+					self._pool = ConnectionPool(
+							host=self._address,
+							user=self._username,
+							passwd=self._password,
+							db=self._database,
+							use_unicode=True,
+							charset=self._databaseCharset,
+							pool_size=self._connectionPoolSize,
+							max_overflow=self._connectionPoolMaxOverflow,
+							timeout=self._connectionPoolTimeout,
+							conv=conv
+					)
+				except Exception as error:
+					logger.logException(error)
+
+					if tryNumber < 2:
+						secondsToWait = 5
+						logger.debug(
+							u"We are waiting {0} seconds before trying "
+							u"an reconnect.".format(secondsToWait)
+						)
+						for _ in range(secondsToWait * 10):
+							time.sleep(0.1)
+						continue
+
+					raise BackendIOError(u"Failed to connect to database '%s' address '%s': %s" % (self._database, self._address, error))
 		finally:
 			self._transactionLock.release()
 
@@ -214,9 +231,9 @@ class MySQL(SQL):
 			self._transactionLock.release()
 
 	def getSet(self, query):
-		logger.debug2(u"getSet: %s" % query)
+		logger.debug2(u"getSet: {0}", query)
 		(conn, cursor) = self.connect()
-		valueSet = []
+
 		try:
 			try:
 				self.execute(query, conn, cursor)
@@ -225,19 +242,19 @@ class MySQL(SQL):
 				if e[0] != 2006:
 					# 2006: MySQL server has gone away
 					raise
+
 				self._createConnectionPool()
 				(conn, cursor) = self.connect()
 				self.execute(query, conn, cursor)
+
 			valueSet = cursor.fetchall()
-			if not valueSet:
-				logger.debug(u"No result for query '%s'" % query)
-				valueSet = []
 		finally:
 			self.close(conn, cursor)
-		return valueSet
+
+		return valueSet or []
 
 	def getRows(self, query):
-		logger.debug2(u"getRows: %s" % query)
+		logger.debug2(u"getRows: {0}", query)
 		onlyAllowSelect(query)
 
 		(conn, cursor) = self.connect(cursorType=MySQLdb.cursors.Cursor)
@@ -256,7 +273,7 @@ class MySQL(SQL):
 
 			valueSet = cursor.fetchall()
 			if not valueSet:
-				logger.debug(u"No result for query '%s'" % query)
+				logger.debug(u"No result for query {0!r}", query)
 				valueSet = []
 		finally:
 			self.close(conn, cursor)
@@ -264,7 +281,7 @@ class MySQL(SQL):
 		return valueSet
 
 	def getRow(self, query, conn=None, cursor=None):
-		logger.debug2(u"getRow: %s" % query)
+		logger.debug2(u"getRow: {0}", query)
 		closeConnection = True
 		if conn and cursor:
 			logger.debug(u"TRANSACTION: conn and cursor given, so we should not close the connection.")
@@ -276,7 +293,7 @@ class MySQL(SQL):
 			try:
 				self.execute(query, conn, cursor)
 			except Exception as e:
-				logger.debug(u"Execute error: %s" % e)
+				logger.debug(u"Execute error: {0!r}", e)
 				if e[0] != 2006:
 					# 2006: MySQL server has gone away
 					raise
@@ -285,10 +302,10 @@ class MySQL(SQL):
 				self.execute(query, conn, cursor)
 			row = cursor.fetchone()
 			if not row:
-				logger.debug(u"No result for query '%s'" % query)
+				logger.debug(u"No result for query {0!r}", query)
 				row = {}
 			else:
-				logger.debug2(u"Result: '%s'" % row)
+				logger.debug2(u"Result: {0!r}", row)
 		finally:
 			if closeConnection:
 				self.close(conn, cursor)
@@ -309,24 +326,24 @@ class MySQL(SQL):
 				colNames.append(u"`{0}`".format(key))
 				if value is None:
 					values.append(u"NULL")
-				elif type(value) is bool:
+				elif isinstance(value, bool):
 					if value:
 						values.append(u"1")
 					else:
 						values.append(u"0")
-				elif type(value) in (float, long, int):
+				elif isinstance(value, (float, long, int)):
 					values.append(u"{0}".format(value))
-				elif type(value) is str:
+				elif isinstance(value, str):
 					values.append(u"\'{0}\'".format(self.escapeApostrophe(self.escapeBackslash(value.decode("utf-8")))))
 				else:
 					values.append(u"\'{0}\'".format(self.escapeApostrophe(self.escapeBackslash(value))))
 
 			query = u'INSERT INTO `{0}` ({1}) VALUES ({2});'.format(table, ', '.join(colNames), ', '.join(values))
-			logger.debug2(u"insert: %s" % query)
+			logger.debug2(u"insert: {0}", query)
 			try:
 				self.execute(query, conn, cursor)
 			except Exception as e:
-				logger.debug(u"Execute error: %s" % e)
+				logger.debug(u"Execute error: {0!r}", e)
 				if e[0] != 2006:
 					# 2006: MySQL server has gone away
 					raise
@@ -352,14 +369,14 @@ class MySQL(SQL):
 						continue
 
 					value = u"NULL"
-				elif type(value) is bool:
+				elif isinstance(value, bool):
 					if value:
 						value = u"1"
 					else:
 						value = u"0"
-				elif type(value) in (float, long, int):
+				elif isinstance(value, (float, long, int)):
 					value = u"%s" % value
-				elif type(value) is str:
+				elif isinstance(value, str):
 					value = u"\'{0}\'".format(self.escapeApostrophe(self.escapeBackslash(value.decode("utf-8"))))
 				else:
 					value = u"\'{0}\'".format(self.escapeApostrophe(self.escapeBackslash(value)))
@@ -367,11 +384,11 @@ class MySQL(SQL):
 				query.append(u"`{0}` = {1}".format(key, value))
 
 			query = u"UPDATE `{0}` SET {1} WHERE {2};".format(table, ', '.join(query), where)
-			logger.debug2(u"update: %s" % query)
+			logger.debug2(u"update: {0}", query)
 			try:
 				self.execute(query, conn, cursor)
 			except Exception as e:
-				logger.debug(u"Execute error: %s" % e)
+				logger.debug(u"Execute error: {0!r}", e)
 				if e[0] != 2006:
 					# 2006: MySQL server has gone away
 					raise
@@ -384,42 +401,47 @@ class MySQL(SQL):
 		return result
 
 	def delete(self, table, where, conn=None, cursor=None):
-		closeConnection = True
 		if conn and cursor:
 			logger.debug(u"TRANSACTION: conn and cursor given, so we should not close the connection.")
 			closeConnection = False
 		else:
+			closeConnection = True
 			(conn, cursor) = self.connect()
+
 		result = 0
 		try:
 			query = u"DELETE FROM `%s` WHERE %s;" % (table, where)
-			logger.debug2(u"delete: %s" % query)
+			logger.debug2(u"delete: {0}", query)
 			try:
 				self.execute(query, conn, cursor)
 			except Exception as e:
-				logger.debug(u"Execute error: %s" % e)
+				logger.debug(u"Execute error: {0}", e)
 				if e[0] != 2006:
 					# 2006: MySQL server has gone away
 					raise
+
 				self._createConnectionPool()
-				(conn, cursor) = self.connect()
+				conn, cursor = self.connect()
 				self.execute(query, conn, cursor)
+
 			result = cursor.rowcount
 		finally:
 			if closeConnection:
 				self.close(conn, cursor)
+
 		return result
 
 	def execute(self, query, conn=None, cursor=None):
-		res = None
-		needClose = False
-		if not conn or not cursor:
-			(conn, cursor) = self.connect()
+		if conn and cursor:
+			needClose = False
+		else:
 			needClose = True
+			conn, cursor = self.connect()
 
+		res = None
 		try:
 			query = forceUnicode(query)
-			logger.debug2(u"SQL query: %s" % query)
+			logger.debug2(u"SQL query: {0}", query)
 			res = cursor.execute(query)
 			if self.doCommit:
 				conn.commit()
@@ -621,12 +643,12 @@ class MySQLBackend(SQLBackend):
 				else:
 					myPPVdefault = u"`isDefault` = 0"
 
-				if type(value) is bool:
+				if isinstance(value, bool):
 					if value:
 						myPPVvalue = u"`value` = 1"
 					else:
 						myPPVvalue = u"`value` = 0"
-				elif type(value) in (float, long, int):
+				elif isinstance(value, (float, long, int)):
 					myPPVvalue = u"`value` = %s" % (value)
 				else:
 					myPPVvalue = u"`value` = '%s'" % (self._sql.escapeApostrophe(self._sql.escapeBackslash(value)))
