@@ -6,7 +6,7 @@ opsi python library - Windows
 This module is part of the desktop management solution opsi
 (open pc server integration) http://www.opsi.org
 
-Copyright (C) 2013-2015 uib GmbH
+Copyright (C) 2013-2016 uib GmbH
 
 http://www.uib.de/
 
@@ -30,8 +30,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 :license: GNU Affero GPL version 3
 """
 
-__version__ = '4.0.6.1'
-
 import difflib
 import locale
 import os
@@ -69,6 +67,8 @@ from datetime import datetime
 from OPSI.Logger import *
 from OPSI.Types import *
 
+__version__ = '4.0.6.30'
+
 logger = Logger()
 hooks = []
 
@@ -195,25 +195,34 @@ def getOpsiHotfixName(helper=None):
 			try:
 				result = execute(helper, shell=False)
 				minor = int(result[0].split(".")[1])
-			except:
+				if int(result[0].split(".")[0]) == 10:
+					logger.notice("Windows 10 detected, changing major from 6 to 10")
+					major = 10
+			except Exception:
 				logger.warning(u"MSHotfix fix for Windows 8.1 don't work. Fallback to normal mode.")
-		if (minor == 0):
-			os = u'vista-win2008'
-		elif (minor == 1):
-			if (arch == 'x86'):
-				os = u'win7'
+			if (major == 10):
+				if (arch == 'x86'):
+					os = u'win10'
+				else:
+					os = u'win10-win2016'
 			else:
-				os = u'win7-win2008r2'
-		elif (minor == 2):
-			if (arch == 'x86'):
-				os = u'win8'
-			else:
-				os = u'win8-win2012'
-		elif (minor == 3):
-			if (arch == 'x86'):
-				os = u'win81'
-			else:
-				os = u'win81-win2012r2'
+				if (minor == 0):
+					os = u'vista-win2008'
+				elif (minor == 1):
+					if (arch == 'x86'):
+						os = u'win7'
+					else:
+						os = u'win7-win2008r2'
+				elif (minor == 2):
+					if (arch == 'x86'):
+						os = u'win8'
+					else:
+						os = u'win8-win2012'
+				elif (minor == 3):
+					if (arch == 'x86'):
+						os = u'win81'
+					else:
+						os = u'win81-win2012r2'
 
 	return u'mshotfix-%s-%s-%s' % (os, arch, lang)
 
@@ -317,6 +326,12 @@ def getDefaultNetworkInterfaceName():
 			return interface.description
 	return None
 
+def getSystemProxySetting():
+	#TODO read proxy settings from system registry
+	#HINTS: If proxycfg is not installed read this way (you have to cut)
+	#netsh winhttp show proxy
+	return None
+
 
 class NetworkPerformanceCounter(threading.Thread):
 	def __init__(self, interface):
@@ -439,7 +454,7 @@ class NetworkPerformanceCounterWMI(threading.Thread):
 			try:
 				import pythoncom
 				pythoncom.CoUninitialize()
-			except:
+			except Exception:
 				pass
 
 	def _getStatistics(self):
@@ -624,7 +639,7 @@ def getRegistryValue(key, subKey, valueName, reflection=True):
 def setRegistryValue(key, subKey, valueName, value):
 	_winreg.CreateKey(key, subKey)
 	hkey = _winreg.OpenKey(key, subKey, 0, _winreg.KEY_WRITE)
-	if type(value) is int:
+	if isinstance(value, int):
 		_winreg.SetValueEx(hkey, valueName, 0, _winreg.REG_DWORD, value)
 	else:
 		_winreg.SetValueEx(hkey, valueName, 0, _winreg.REG_SZ, value)
@@ -941,7 +956,7 @@ def getSessionInformation(sessionId, winApiBugCommand = None):
 	wtsUserName = None
 	try:
 		wtsUserName = win32ts.WTSQuerySessionInformation(None, sessionId, win32ts.WTSUserName)
-	except:
+	except Exception:
 		pass
 
 	# 'UserName': u'Administrator',
@@ -964,7 +979,7 @@ def getSessionInformation(sessionId, winApiBugCommand = None):
 				try:
 					if wtsUserName and not sessionData['UserName'].lower() ==  wtsUserName.lower():
 						sessionData['UserName'] = wtsUserName
-				except:
+				except Exception:
 					pass
 				logger.debug(u"   Found session: %s" % sessionData)
 				return sessionData
@@ -1391,7 +1406,7 @@ def execute(cmd, waitForEnding=True, getHandle=False, ignoreExitCode=[], exitOnS
 				if (timeout > 0) and (time.time() - startTime >= timeout):
 					try:
 						proc.kill()
-					except:
+					except Exception:
 						pass
 					raise IOError(exitCode, u"Command '%s' timed out atfer %d seconds" % (cmd, (time.time() - startTime)) )
 
@@ -1413,9 +1428,9 @@ def execute(cmd, waitForEnding=True, getHandle=False, ignoreExitCode=[], exitOnS
 
 	logger.debug(u"Exit code: %s" % exitCode)
 	if exitCode:
-		if   type(ignoreExitCode) is bool and ignoreExitCode:
+		if isinstance(ignoreExitCode, bool) and ignoreExitCode:
 			pass
-		elif type(ignoreExitCode) is list and exitCode in ignoreExitCode:
+		elif isinstance(ignoreExitCode, list) and exitCode in ignoreExitCode:
 			pass
 		else:
 			raise IOError(exitCode, u"Command '%s' failed (%s):\n%s" % (cmd, exitCode, u'\n'.join(result)))
@@ -1446,7 +1461,7 @@ def getPids(process, sessionId=None):
 		sid = u'unknown'
 		try:
 			sid = win32ts.ProcessIdToSessionId(pid)
-		except:
+		except Exception:
 			pass
 		logger.debug2(u"   got process %s with pid %d in session %s" % (pe32.szExeFile, pid, sid))
 		if (pe32.szExeFile.lower() == process.lower()):
@@ -1767,7 +1782,7 @@ def setLocalSystemTime(timestring):
 	try:
 		dt = datetime.datetime.strptime(timestring, '%Y-%m-%d %H:%M:%S.%f')
 		logger.info(u"Setting Systemtime Time to %s" % timestring)
-		winapi32.SetSystemTime(dt.year, dt.month, dt.weekday(), dt.day, dt.hour, dt.minute, dt.second, dt.microsecond)
+		win32api.SetSystemTime(dt.year, dt.month, 0, dt.day, dt.hour, dt.minute, dt.second, 0)
 	except Exception as e:
 		logger.error(u"Failed to set System Time: '%s'" % e)
 
@@ -1909,7 +1924,6 @@ class Impersonate:
 		s.lpDesktop = self.winsta + u'\\' + self.desktop
 
 		logger.notice(u"Running command '%s' as user '%s' on desktop '%s'" % (command, self.username, self.desktop))
-		self.userEnvironment
 		(hProcess, hThread, dwProcessId, dwThreadId) = win32process.CreateProcessAsUser(
 					self.userToken, None, command, None, None, 0, dwCreationFlags, self.userEnvironment, None, s)
 		logger.info(u"Process startet, pid: %d" % dwProcessId)
