@@ -1,56 +1,70 @@
-#!/usr/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
+
+# This module is part of the desktop management solution opsi
+# Based on urllib3
+# (open pc server integration) http://www.opsi.org
+# Copyright (C) 2010 Andrey Petrov
+# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-   = = = = = = = = = = = = = = = = = =
-   =   opsi python library - HTTP    =
-   = = = = = = = = = = = = = = = = = =
+opsi python library - HTTP
 
-   This module is part of the desktop management solution opsi
-   Based on urllib3
-   (open pc server integration) http://www.opsi.org
 
-   Copyright (C) 2010 Andrey Petrov
-   Copyright (C) 2010-2014 uib GmbH
+.. versionadded:: 4.0.6.9
 
-   http://www.uib.de/
+  Added functions :py:func:`deflateEncode`, :py:func:`deflateDecode`,
+  :py:func:`gzipEncode` and :py:func:`gzipDecode`.
 
-   All rights reserved.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2 as
-   published by the Free Software Foundation.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-   @copyright:	uib GmbH <info@uib.de>
-   @author: Jan Schneider <j.schneider@uib.de>
-   @license: GNU General Public License version 2
+:author: Jan Schneider <j.schneider@uib.de>
+:author: Niko Wenselowski <n.wenselowski@uib.de>
+:author: Erol Ueluekmen <e.ueluekmen@uib.de>
+:license: GNU Affero General Public License version 3
 """
 
 import base64
+import gzip
 import os
 import random
 import re
 import socket
 import time
+import zlib
+import urlparse
+from contextlib import closing  # Needed for Python 2.6
+from contextlib import contextmanager
+from io import BytesIO
+
+try:
+	from cStringIO import StringIO
+except ImportError:
+	from io import StringIO
+
 from Queue import Queue, Empty, Full
 from httplib import HTTPConnection, HTTPSConnection, HTTPException
 from socket import error as SocketError, timeout as SocketTimeout
 import ssl as ssl_module
 from OpenSSL import crypto
 
-from OPSI.Types import *
+from OPSI.Types import forceBool, forceFilename, forceInt, forceUnicode, forceUnicodeLower
+from OPSI.Types import OpsiTimeoutError, OpsiServiceVerificationError
 from OPSI.Logger import LOG_DEBUG, LOG_INFO, Logger
 from OPSI.Util import encryptWithPublicKeyFromX509CertificatePEMFile, randomString
-logger = Logger()
 
+logger = Logger()
 
 connectionPools = {}
 totalRequests = 0
@@ -58,6 +72,14 @@ totalRequests = 0
 # This could be an import - but support for pycurl is currently not fully implrement
 pycurl = None
 
+if hasattr(ssl_module, '_create_unverified_context'):
+	# We are running a new version of Python that implements PEP 476:
+	# https://www.python.org/dev/peps/pep-0476/
+	# To not break our expected behaviour we patch the default context
+	# until we have a correct certificate check implementation.
+	# TODO: remove this workaround when support for TLS1.1+ is implemented
+	ssl_module._create_default_https_context = ssl_module._create_unverified_context
+
 
 def hybi10Encode(data):
 	# Code stolen from http://lemmingzshadow.net/files/2011/09/Connection.php.txt
@@ -76,12 +98,12 @@ def hybi10Encode(data):
 		frame.append(dataLength & 0xff)
 
 	frame.extend(mask)
-	for i in range(len(data)):
-		frame.append(ord(data[i]) ^ mask[i % 4])
+	for i, currentData in enumerate(data):
+		frame.append(ord(currentData) ^ mask[i % 4])
 
 	encodedData = ''
-	for i in range(len(frame)):
-		encodedData += chr(frame[i])
+	for currentFrame in frame:
+		encodedData += chr(currentFrame)
 	return encodedData
 
 
@@ -110,8 +132,9 @@ def hybi10Decode(data):
 		else:
 			mask = data[2:6]
 			codedData = data[6:]
-		for i in range(len(codedData)):
-			decodedData += chr( ord(codedData[i]) ^ ord(mask[i % 4]) )
+
+		for i, currentData in enumerate(codedData):
+			decodedData += chr(ord(currentData) ^ ord(mask[i % 4]))
 	else:
 		if dataLength == 126:
 			decodedData = data[4:]
@@ -119,6 +142,7 @@ def hybi10Decode(data):
 			decodedData = data[10:]
 		else:
 			decodedData = data[2:]
+
 	return decodedData
 
 
@@ -134,14 +158,14 @@ def non_blocking_connect_http(self, connectTimeout=0):
 				raise OpsiTimeoutError(u"Timed out after {0:d} seconds (last error: {1})".format(connectTimeout, forceUnicode(lastError)))
 			sock.connect((self.host, self.port))
 			break
-		except socket.error as e:
-			logger.logException(e, LOG_DEBUG)
-			logger.debug(e)
-			if e[0] in (106, 10056):
+		except socket.error as error:
+			logger.logException(error, LOG_DEBUG)
+			logger.debug(error)
+			if error[0] in (106, 10056):
 				# Transport endpoint is already connected
 				break
-			if e[0] not in (114, ) or not lastError:
-				lastError = e
+			if error[0] not in (114, ) or not lastError:
+				lastError = error
 			time.sleep(0.5)
 	sock.settimeout(None)
 	self.sock = sock
@@ -165,8 +189,8 @@ def getPeerCertificate(httpsConnectionOrSSLSocket, asPEM=True):
 		if not asPEM:
 			return cert
 		return crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
-	except Exception as e:
-		logger.debug(u"Failed to get peer cert: %s" % e)
+	except Exception as error:
+		logger.debug(u"Failed to get peer cert: %s" % error)
 		return None
 
 
@@ -192,12 +216,12 @@ class HTTPResponse(object):
 	Similar to httplib's HTTPResponse but the data is pre-loaded.
 	"""
 	def __init__(self, data='', headers={}, status=0, version=0, reason=None, strict=0):
-		self.data    = data
+		self.data = data
 		self.headers = headers
-		self.status  = status
+		self.status = status
 		self.version = version
-		self.reason  = reason
-		self.strict  = strict
+		self.reason = reason
+		self.strict = strict
 
 	def addData(self, data):
 		self.data += data
@@ -217,17 +241,17 @@ class HTTPResponse(object):
 			except Exception:
 				pass
 		elif header.count(':') > 0:
-			(k, v) = header.split(':', 1)
-			k = k.lower().strip()
-			v = v.strip()
-			if k == 'content-length':
+			(key, value) = header.split(':', 1)
+			key = key.lower().strip()
+			value = value.strip()
+			if key == 'content-length':
 				try:
-					v = int(v)
-					if v < 0:
-						v = 0
-				except:
+					value = int(value)
+					if value < 0:
+						value = 0
+				except Exception:
 					return
-			self.headers[k] = v
+			self.headers[key] = value
 
 	@staticmethod
 	def from_httplib(r):
@@ -238,6 +262,7 @@ class HTTPResponse(object):
 		NOTE: This method will perform r.read() which will have side effects
 		on the original http.HTTPResponse object.
 		"""
+		logger.debug2("Creating HTTPResponse from httplib...")
 		return HTTPResponse(
 			data=r.read(),
 			headers=dict(r.getheaders()),
@@ -254,29 +279,34 @@ class HTTPResponse(object):
 	def getheader(self, name, default=None):
 		return self.headers.get(name, default)
 
+
 class HTTPConnectionPool(object):
 
 	scheme = 'http'
 
-	def __init__(self, host, port, socketTimeout=None, connectTimeout=None, retryTime=0, maxsize=1, block=False, reuseConnection=False, verifyServerCert=False, serverCertFile=None, caCertFile=None, verifyServerCertByCa=False):
+	def __init__(self, host, port, socketTimeout=None, connectTimeout=None,
+				retryTime=0, maxsize=1, block=False, reuseConnection=False,
+				verifyServerCert=False, serverCertFile=None, caCertFile=None,
+				verifyServerCertByCa=False, proxyURL=None):
 
-		self.host                 = forceUnicode(host)
-		self.port                 = forceInt(port)
-		self.socketTimeout        = forceInt(socketTimeout or 0)
-		self.connectTimeout       = forceInt(connectTimeout or 0)
-		self.retryTime            = forceInt(retryTime)
-		self.block                = forceBool(block)
-		self.reuseConnection      = forceBool(reuseConnection)
-		self.pool                 = None
-		self.usageCount           = 1
-		self.num_connections      = 0
-		self.num_requests         = 0
-		self.httplibDebugLevel    = 0
-		self.peerCertificate      = None
-		self.serverVerified       = False
-		self.verifyServerCert     = False
-		self.serverCertFile       = None
-		self.caCertFile           = None
+		self.host = forceUnicode(host)
+		self.port = forceInt(port)
+		self.socketTimeout = forceInt(socketTimeout or 0)
+		self.connectTimeout = forceInt(connectTimeout or 0)
+		self.retryTime = forceInt(retryTime)
+		self.block = forceBool(block)
+		self.reuseConnection = forceBool(reuseConnection)
+		self.proxyURL = forceUnicode(proxyURL or u"")
+		self.pool = None
+		self.usageCount = 1
+		self.num_connections = 0
+		self.num_requests = 0
+		self.httplibDebugLevel = 0
+		self.peerCertificate = None
+		self.serverVerified = False
+		self.verifyServerCert = False
+		self.serverCertFile = None
+		self.caCertFile = None
 		self.verifyServerCertByCa = False
 
 		if isinstance(self, HTTPSConnectionPool):
@@ -318,12 +348,8 @@ class HTTPConnectionPool(object):
 				try:
 					conn = self.pool.get(block=False)
 					if conn:
-						try:
-							if conn.sock:
-								conn.sock.close()
-							conn.close()
-						except:
-							pass
+						closeConnection(conn)
+
 					time.sleep(0.001)
 				except Empty:
 					break
@@ -345,10 +371,28 @@ class HTTPConnectionPool(object):
 		Return a fresh HTTPConnection.
 		"""
 		self.num_connections += 1
-		logger.debug(u"Starting new HTTP connection (%d) to %s:%d" % (self.num_connections, self.host, self.port))
-		conn = HTTPConnection(host=self.host, port=self.port)
-		non_blocking_connect_http(conn, self.connectTimeout)
-		logger.debug(u"Connection established to: %s" % self.host)
+		if self.proxyURL:
+			headers = {}
+			try:
+				url = urlparse.urlparse(self.proxyURL)
+				if url.password:
+					logger.setConfidentialStrings(url.password)
+					logger.debug(u"Starting new HTTP connection (%d) to %s:%d over proxy-url %s" % (self.num_connections, self.host, self.port, self.proxyURL))
+
+				conn = HTTPConnection(host=url.hostname, port=url.port)
+				if url.username and url.password:
+					logger.debug(u"Proxy Authentication detected, setting auth with user: '%s'" % url.username)
+					auth = "{username}:{password}".format(username=url.username, password=url.password)
+					headers['Proxy-Authorization'] = 'Basic ' + base64.base64encode(auth)
+				conn.set_tunnel(self.host, self.port, headers)
+				logger.debug(u"Connection established to: %s" % self.host)
+			except Exception as error:
+				logger.error(error)
+		else:
+			logger.debug(u"Starting new HTTP connection (%d) to %s:%d" % (self.num_connections, self.host, self.port))
+			conn = HTTPConnection(host=self.host, port=self.port)
+			non_blocking_connect_http(conn, self.connectTimeout)
+			logger.debug(u"Connection established to: %s" % self.host)
 		return conn
 
 	def _get_conn(self, timeout=None):
@@ -356,11 +400,11 @@ class HTTPConnectionPool(object):
 		Get a connection. Will return a pooled connection if one is available.
 		Otherwise, a fresh connection is returned.
 		"""
-		conn = None
 		try:
 			conn = self.pool.get(block=self.block, timeout=timeout)
 		except Empty:
-			pass  # Oh well, we'll create a new connection then
+			# Oh well, we'll create a new connection then
+			conn = None
 
 		return conn or self._new_conn()
 
@@ -430,7 +474,6 @@ class HTTPConnectionPool(object):
 			firstTryTime = now
 
 		conn = None
-		# Check host
 		if assert_same_host and not self.is_same_host(url):
 			host = "%s://%s" % (self.scheme, self.host)
 			if self.port:
@@ -438,18 +481,15 @@ class HTTPConnectionPool(object):
 			raise HostChangedError(u"Connection pool with host '%s' tried to open a foreign host: %s" % (host, url))
 
 		try:
-			# Request a connection from the queue
 			conn = self._get_conn()
 
 			if self.httplibDebugLevel:
 				conn.set_debuglevel(self.httplibDebugLevel)
 
-			# Make the request
 			self.num_requests += 1
 
 			global totalRequests
 			totalRequests += 1
-			#logger.essential("totalRequests: %d" % totalRequests)
 
 			randomKey = None
 			if isinstance(self, HTTPSConnectionPool) and self.verifyServerCert and not self.serverVerified:
@@ -457,26 +497,26 @@ class HTTPConnectionPool(object):
 					logger.info(u"Encoding authorization")
 					randomKey = randomString(32).encode('latin-1')
 					encryptedKey = encryptWithPublicKeyFromX509CertificatePEMFile(randomKey, self.serverCertFile)
-					headers['X-opsi-service-verification-key'] = base64.encodestring(encryptedKey)
-					for (key, value) in headers.items():
-						if (key.lower() == 'authorization'):
+					headers['X-opsi-service-verification-key'] = base64.b64encode(encryptedKey)
+					for key, value in headers.items():
+						if key.lower() == 'authorization':
 							if value.lower().startswith('basic'):
 								value = value[5:].strip()
-							value = base64.decodestring(value).strip()
+							value = base64.b64decode(value).strip()
 							encodedAuth = encryptWithPublicKeyFromX509CertificatePEMFile(value, self.serverCertFile)
-							headers[key] = 'Opsi ' + base64.encodestring(encodedAuth).strip()
-				except Exception as e:
-					logger.logException(e, LOG_INFO)
-					logger.critical(u"Cannot verify server based on certificate file '%s': %s" % (self.serverCertFile, e))
+							headers[key] = 'Opsi ' + base64.b64encode(encodedAuth)
+				except Exception as error:
+					logger.logException(error, LOG_INFO)
+					logger.critical(u"Cannot verify server based on certificate file {0!r}: {1}", self.serverCertFile, error)
 					randomKey = None
 
+			logger.debug2("Handing data to connection...")
 			conn.request(method, url, body=body, headers=headers)
 			if self.socketTimeout:
 				conn.sock.settimeout(self.socketTimeout)
 			else:
 				conn.sock.settimeout(None)
 			httplib_response = conn.getresponse()
-			#logger.debug(u"\"%s %s %s\" %s %s" % (method, url, conn._http_vsn_str, httplib_response.status, httplib_response.length))
 
 			# from_httplib will perform httplib_response.read() which will have
 			# the side effect of letting us use this connection for another
@@ -484,90 +524,79 @@ class HTTPConnectionPool(object):
 			response = HTTPResponse.from_httplib(httplib_response)
 
 			if randomKey:
+				logger.debug2("Checking for random key...")
 				try:
 					key = response.getheader('x-opsi-service-verification-key', None)
 					if not key:
 						raise Exception(u"HTTP header 'X-opsi-service-verification-key' missing")
-					if (key.strip() != randomKey.strip()):
+					if key.strip() != randomKey.strip():
 						raise Exception(u"opsi-service-verification-key '%s' != '%s'" % (key, randomKey))
 					self.serverVerified = True
 					logger.notice(u"Service verified by opsi-service-verification-key")
-				except Exception as e:
-					logger.error(u"Service verification failed: %s" % e)
-					raise OpsiServiceVerificationError(u"Service verification failed: %s" % e)
+				except Exception as error:
+					logger.error(u"Service verification failed: {0}", error)
+					raise OpsiServiceVerificationError(u"Service verification failed: %s" % error)
 
 			if self.serverCertFile and self.peerCertificate:
 				try:
 					certDir = os.path.dirname(self.serverCertFile)
 					if not os.path.exists(certDir):
 						os.makedirs(certDir)
-					f = open(self.serverCertFile, 'w')
-					f.write(self.peerCertificate)
-					f.close()
-				except Exception as e:
-					logger.error(u"Failed to create server cert file '%s': %s" % (self.serverCertFile, e))
+					with open(self.serverCertFile, 'w') as f:
+						f.write(self.peerCertificate)
+				except Exception as error:
+					logger.error(u"Failed to create server cert file {0!r}: {1}", self.serverCertFile, error)
 
 			# Put the connection back to be reused
 			if self.reuseConnection:
 				self._put_conn(conn)
 			else:
-				logger.debug(u"Closing connection: %s" % conn)
+				logger.debug(u"Closing connection: {0}", conn)
 				self._put_conn(None)
-				try:
-					if conn.sock:
-						conn.sock.close()
-					conn.close()
-				except:
-					pass
-
-		except (SocketTimeout, Empty, HTTPException, SocketError) as e:
+				closeConnection(conn)
+		except (SocketTimeout, Empty, HTTPException, SocketError) as error:
+			logger.logException(error, logLevel=LOG_DEBUG)
 			try:
-				logger.debug(u"Request to host '%s' failed, retry: %s, firstTryTime: %s, now: %s, retryTime: %s, connectTimeout: %s, socketTimeout: %s (%s)" \
-					% (self.host, retry, firstTryTime, now, self.retryTime, self.connectTimeout, self.socketTimeout, forceUnicode(e)))
-			except:
+				logger.debug(
+					u"Request to host {0!r} failed, retry: {1}, firstTryTime: {2}, now: {3}, retryTime: {4}, connectTimeout: {5}, socketTimeout: {6} ({7!r})",
+					self.host, retry, firstTryTime, now, self.retryTime, self.connectTimeout, self.socketTimeout, error
+				)
+			except Exception as loggingError:
+				logger.debug(u"Logging exception failed: {0}", forceUnicode(loggingError))
+				logger.debug(u"Trying to log again without original exception.")
 				try:
-					logger.debug(u"Request to host '%s' failed, retry: %s, firstTryTime: %s, now: %s, retryTime: %s, connectTimeout: %s, socketTimeout: %s" \
-						% (self.host, retry, firstTryTime, now, self.retryTime, self.connectTimeout, self.socketTimeout))
-				except:
-					pass
+					logger.debug(
+						u"Request to host {0!r} failed, retry: {1}, firstTryTime: {2}, now: {3}, retryTime: {4}, connectTimeout: {5}, socketTimeout: {6}",
+						self.host, retry, firstTryTime, now, self.retryTime, self.connectTimeout, self.isocketTimeout
+					)
+				except Exception as error:
+					logger.debug(u"Logging message failed: {0!r}", error)
+					logger.warning(u"Logging message failed: {0}", forceUnicode(error))
 
 			self._put_conn(None)
-			try:
-				if conn:
-					if conn.sock:
-						conn.sock.close()
-					conn.close()
-			except:
-				pass
+			closeConnection(conn)
+
 			if retry and (now - firstTryTime < self.retryTime):
-				logger.debug(u"Request to '%s' failed: %s, retrying" % (self.host, forceUnicode(e)))
-				time.sleep(0.1)
+				logger.debug(u"Request to {0!r} failed: {1}", self.host, forceUnicode(error))
+				logger.debug(u"Waiting before retry...")
+				time.sleep(0.2)
 				return self.urlopen(method, url, body, headers, retry, redirect, assert_same_host, firstTryTime)
 			else:
+				if retry:
+					logger.warning("Connecting to {0!r} did not succeed after retrying.", self.host)
+
 				raise
 		except Exception:
 			self._put_conn(None)
-			try:
-				if conn:
-					if conn.sock:
-						conn.sock.close()
-					conn.close()
-			except:
-				pass
+			closeConnection(conn)
 			raise
 
 		# Handle redirection
-		if redirect and response.status in (301, 302, 303, 307) and 'location' in response.headers: # Redirect, retry
+		if redirect and response.status in (301, 302, 303, 307) and 'location' in response.headers:
 			logger.info(u"Redirecting %s -> %s" % (url, response.headers.get('location')))
 			time.sleep(0.1)
 			self._put_conn(None)
-			try:
-				if conn:
-					if conn.sock:
-						conn.sock.close()
-					conn.close()
-			except:
-				pass
+			closeConnection(conn)
 			return self.urlopen(method, url, body, headers, retry, redirect, assert_same_host, firstTryTime)
 
 		return response
@@ -584,20 +613,39 @@ class HTTPSConnectionPool(HTTPConnectionPool):
 		"""
 		Return a fresh HTTPSConnection.
 		"""
-		logger.debug(u"Starting new HTTPS connection (%d) to %s:%d" % (self.num_connections, self.host, self.port))
-		conn = HTTPSConnection(host = self.host, port = self.port)
+
+		if self.proxyURL:
+			headers = {}
+			try:
+				url = urlparse.urlparse(self.proxyURL)
+				if url.password:
+					logger.setConfidentialString(url.password)
+					logger.debug(u"Starting new HTTPS connection (%d) to %s:%d over proxy-url %s" % (self.num_connections, self.host, self.port, self.proxyURL))
+				conn = HTTPSConnection(host=url.hostname, port=url.port)
+				if url.username and url.password:
+					logger.debug(u"Proxy Authentication detected, setting auth with user: '%s'" % url.username)
+					auth = "{username}:{password}".format(username=url.username,password=url.password)
+					headers['Proxy-Authorization'] = 'Basic ' + base64.base64encode(auth)
+				conn.set_tunnel(self.host, self.port, headers)
+				logger.debug(u"Connection established to: %s" % self.host)
+			except Exception as e:
+				logger.logException(e)
+		else:
+			logger.debug(u"Starting new HTTPS connection (%d) to %s:%d" % (self.num_connections, self.host, self.port))
+			conn = HTTPSConnection(host=self.host, port=self.port)
+			logger.debug(u"Connection established to: %s" % self.host)
+
 		if self.verifyServerCert or self.verifyServerCertByCa:
 			try:
 				non_blocking_connect_https(conn, self.connectTimeout, self.caCertFile)
 				if not self.verifyServerCertByCa:
 					self.serverVerified = True
-			except Exception as e:
-				logger.debug(e)
-				if (e.__class__.__name__ != 'SSLError') or self.verifyServerCertByCa:
-					raise OpsiServiceVerificationError(u"Failed to verify server cert by CA: %s" % e)
+			except Exception as error:
+				logger.debug(error)
+				if error.__class__.__name__ != 'SSLError' or self.verifyServerCertByCa:
+					raise OpsiServiceVerificationError(u"Failed to verify server cert by CA: %s" % error)
 				non_blocking_connect_https(conn, self.connectTimeout)
 
-		logger.debug(u"Connection established to: %s" % self.host)
 		self.num_connections += 1
 		self.peerCertificate = getPeerCertificate(conn, asPEM=True)
 		if self.verifyServerCertByCa:
@@ -616,9 +664,7 @@ class HTTPSConnectionPool(HTTPConnectionPool):
 				else:
 					raise Exception(u"Failed to get peer certificate")
 			except Exception:
-				if conn.sock:
-					conn.sock.close()
-				conn.close()
+				closeConnection(conn)
 				raise
 		return conn
 
@@ -635,14 +681,17 @@ def urlsplit(url):
 		scheme = scheme.lower()
 	parts = url.split('/', 1)
 	host = parts[0]
+
 	if len(parts) > 1:
 		baseurl += parts[1]
-	if host.find('@') != -1:
-		(username, host) = host.split('@', 1)
-		if (username.find(':') != -1):
-			(username, password) = username.split(':', 1)
-	if host.find(':') != -1:
-		(host, port) = host.split(':', 1)
+
+	if '@' in host:
+		username, host = host.split('@', 1)
+		if ':' in username:
+			username, password = username.split(':', 1)
+
+	if ':' in host:
+		host, port = host.split(':', 1)
 		port = int(port)
 	return (scheme, host, port, baseurl, username, password)
 
@@ -663,6 +712,7 @@ def getSharedConnectionPoolFromUrl(url, **kw):
 			port = 443
 		else:
 			port = 80
+
 	return getSharedConnectionPool(scheme, host, port, **kw)
 
 
@@ -675,12 +725,14 @@ def getSharedConnectionPool(scheme, host, port, **kw):
 		if kw['preferCurl'] and pycurl is not None:
 			curl = True
 		del kw['preferCurl']
+
 	global connectionPools
 	if curl:
 		poolKey = u'curl:%s:%d' % (host, port)
 	else:
 		poolKey = u'httplib:%s:%d' % (host, port)
-	if not connectionPools.has_key(poolKey):
+
+	if poolKey not in connectionPools:
 		if scheme in ('https', 'webdavs'):
 			if curl:
 				connectionPools[poolKey] = CurlHTTPSConnectionPool(host, port=port, **kw)
@@ -696,34 +748,63 @@ def getSharedConnectionPool(scheme, host, port, **kw):
 		maxsize = kw.get('maxsize', 0)
 		if maxsize > connectionPools[poolKey].maxsize:
 			connectionPools[poolKey].adjustSize(maxsize)
+
 	return connectionPools[poolKey]
 
 
 def destroyPool(pool):
 	global connectionPools
-	for (k, p) in connectionPools.items():
-		if (p == pool):
-			del connectionPools[k]
+	for key, poolinstance in connectionPools.items():
+		if poolinstance == pool:
+			del connectionPools[key]
 			break
 
 
-if (__name__ == '__main__'):
-	pass
+def deflateEncode(data, level=1):
+	if isinstance(data, unicode):
+		data = data.encode('utf-8')
+	return zlib.compress(data, level)
 
-	#pool = HTTPSConnectionPool(host = 'download.uib.de', port = 443, connectTimeout=5, caCertFile = '/tmp/xxx', verifyServerCertByCa=True)
-	#resp = pool.urlopen('GET', url = '/index.html', body=None, headers={"accept": "text/html", "user-agent": "test"})
-	#print resp.data
-	#time.sleep(5)
-	#pool = CurlHTTPSConnectionPool(host = 'download.uib.de', port = 443, connectTimeout=5)
-	#resp = pool.urlopen('GET', url = '/index.html', body=None, headers={"accept": "text/html", "user-agent": "test"})
-	#print resp.data
-	#pool = CurlHTTPConnectionPool(host = 'www.uib.de', port = 80, socketTimeout=None, connectTimeout=5, reuseConnection=True)
-	#resp = pool.urlopen('GET', url = '/www/home/index.html', body=None, headers={"accept": "text/html", "user-agent": "test"})
-	#print resp.headers
-	#resp = pool.urlopen('GET', url = '/www/home/index.html', body=None, headers={"accept": "text/html", "user-agent": "test"})
-	#print resp.data
-	#print resp.headers
-	#print resp.status
-	#print resp.version
-	#print resp.reason
-	#print resp.strict
+
+def deflateDecode(data):
+	return forceUnicode(zlib.decompress(data))
+
+
+def gzipEncode(data, level=1):
+	if isinstance(data, unicode):
+		data = data.encode('utf-8')
+
+	inmemoryFile = BytesIO()
+	with closing(gzip.GzipFile(fileobj=inmemoryFile, mode="w", compresslevel=level)) as gzipfile:
+		gzipfile.write(data)
+
+	return inmemoryFile.getvalue()
+
+
+def gzipDecode(data):
+	with closing(gzip.GzipFile(fileobj=StringIO(data), mode="r")) as gzipfile:
+		uncompressedData = gzipfile.read()
+
+	return forceUnicode(uncompressedData)
+
+
+@contextmanager
+def closingConnection(connection):
+	"This contextmanager closes the connection afterwards."
+	try:
+		yield connection
+	finally:
+		closeConnection(connection)
+
+
+def closeConnection(connection):
+	"Close the given connection and any socket that may be open on it."
+	try:
+		connection.sock.close()
+	except Exception:
+		pass
+
+	try:
+		connection.close()
+	except Exception:
+		pass
