@@ -1,37 +1,31 @@
-#!/usr/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
+
+# This file is part of python-opsi.
+# Copyright (C) 2006-2008, 2015 uib GmbH <info@uib.de>
+# All rights reserved.
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 opsi python library - Message
 
-This module is part of the desktop management solution opsi
-(open pc server integration) http://www.opsi.org
+Working with subjects and progress information.
 
-Copyright (C) 2006, 2007, 2008 uib GmbH
-
-http://www.uib.de/
-
-All rights reserved.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-@copyright:	uib GmbH <info@uib.de>
-@author: Jan Schneider <j.schneider@uib.de>
-@license: GNU General Public License version 2
+:author: Jan Schneider <j.schneider@uib.de>
+:license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.1'
-
 import json
 import threading
 import time
@@ -43,6 +37,8 @@ from OPSI.Logger import Logger
 from OPSI.Types import (forceBool, forceInt, forceIntList, forceIpAddress,
 	forceList, forceUnicode, forceUnicodeList)
 
+__version__ = '4.0.6.29'
+
 logger = Logger()
 
 
@@ -535,12 +531,12 @@ class NotificationServerFactory(ServerFactory, SubjectsObserver):
 		self.notify( name = u"endConnection", params = [ clientIds ] )
 
 	def notify(self, name, params, clients = []):
-		if not type(params) is list:
-			params = [ params ]
+		if not isinstance(params, list):
+			params = [params]
 		if not clients:
 			clients = self.clients
-		if not type(clients) is list:
-			clients = [ clients ]
+		if not isinstance(clients, list):
+			clients = [clients]
 		if not clients:
 			logger.debug(u"cannot send notification '%s', no client connected" % name)
 			return
@@ -548,7 +544,7 @@ class NotificationServerFactory(ServerFactory, SubjectsObserver):
 		for client in clients:
 			# json-rpc: notifications have id null
 			jsonString = json.dumps( {"id": None, "method": name, "params": params } )
-			if type(jsonString) is unicode:
+			if isinstance(jsonString, unicode):
 				jsonString = jsonString.encode('utf-8')
 			client.sendLine(jsonString)
 
@@ -703,8 +699,8 @@ class NotificationClientFactory(ClientFactory):
 		logger.debug(u"executing method '%s', params %s" % (method, params))
 		if not params:
 			params = []
-		if not type(params) in (list, tuple):
-			params = [ params ]
+		if not isinstance(params, (list, tuple)):
+			params = [params]
 
 		timeout = 0
 		while not self.isReady() and (timeout < self._timeout):
