@@ -1,10 +1,10 @@
-#!/usr/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # This module is part of the desktop management solution opsi
 # (open pc server integration) http://www.opsi.org
 #
-# Copyright (C) 2006-2010, 2013-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2006-2010, 2013-2015 uib GmbH <info@uib.de>
 # All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,24 +24,25 @@ Functions to work with Windows drivers.
 
 :author: Jan Schneider <j.schneider@uib.de>
 :author: Erol Ueluekmen <e.ueluekmen@uib.de>
+:author: Niko Wenselowski <n.wenselowski@uib.de>
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import os
 import re
 import codecs
 
+import OPSI.System as System
 from OPSI.Logger import Logger
-from OPSI.Types import *
-from OPSI.Object import *
-from OPSI import System
+from OPSI.Object import AuditHardware, AuditHardwareOnHost
+from OPSI.Types import (forceFilename, forceInt, forceList,
+	forceObjectClassList, forceUnicode, forceUnicodeList)
 from OPSI.Util import findFiles
-from OPSI.Util.File import *
-from OPSI.Util.Message import *
+from OPSI.Util.File import InfFile, TxtSetupOemFile
 from OPSI.Util.Repository import Repository
 
+__version__ = '4.0.6.15'
+
 logger = Logger()
 
 
@@ -49,7 +50,7 @@ def searchWindowsDrivers(driverDir, auditHardwares, messageSubject=None, srcRepo
 	driverDir = forceFilename(driverDir)
 	try:
 		auditHardwares = forceObjectClassList(auditHardwares, AuditHardware)
-	except:
+	except Exception:
 		auditHardwares = forceObjectClassList(auditHardwares, AuditHardwareOnHost)
 
 	exists = os.path.exists
@@ -152,13 +153,11 @@ def integrateWindowsDrivers(driverSourceDirectories, driverDestinationDirectory,
 			driversOnMachine[vendorId].append(deviceId)
 
 	exists = os.path.exists
-	listdir = os.listdir
 	copy = System.copy
 	if srcRepository:
 		if not isinstance(srcRepository, Repository):
 			raise Exception(u"Not a repository: %s" % srcRepository)
 		exists = srcRepository.exists
-		listdir = srcRepository.listdir
 		copy = srcRepository.copy
 
 	logger.info(u"Integrating drivers: %s" % driverSourceDirectories)
@@ -275,7 +274,7 @@ def integrateWindowsHardwareDrivers(driverSourceDirectory, driverDestinationDire
 	driverDestinationDirectory = forceFilename(driverDestinationDirectory)
 	try:
 		auditHardwares = forceObjectClassList(auditHardwares, AuditHardware)
-	except:
+	except Exception:
 		auditHardwares = forceObjectClassList(auditHardwares, AuditHardwareOnHost)
 
 	drivers = searchWindowsDrivers(driverDir=driverSourceDirectory, auditHardwares=auditHardwares, messageSubject=messageSubject, srcRepository=srcRepository)
@@ -336,8 +335,9 @@ def integrateWindowsTextmodeDrivers(driverDirectory, destination, devices, sifFi
 			logger.debug2(u"Testing if textmode driver '%s' supports device %s" % (driverPath, device))
 			try:
 				deviceKnown = txtSetupOemFile.isDeviceKnown(vendorId=device.get('vendorId'), deviceId=device.get('deviceId'))
-			except Exception as e:
-				logger.critical(u"Error by integrating TextMode driver, error was: %s" % e)
+			except Exception as error:
+				logger.critical(u"Error by integrating TextMode driver, error was: %s" % error)
+
 			if deviceKnown:
 				logger.debug(u"Textmode driver '%s' supports device %s" % (driverPath, device))
 				supportedDevice = device
@@ -378,22 +378,21 @@ def integrateWindowsTextmodeDrivers(driverDirectory, destination, devices, sifFi
 			massStorageDriverLines = []
 			oemBootFileLines = []
 			section = u''
-			sif = codecs.open(sifFile, 'r', 'cp1250')
-			for line in sif.readlines():
-				if line.strip():
-					logger.debug2(u"Current sif file content: %s" % line.rstrip())
-				if line.strip().startswith(u'['):
-					section = line.strip().lower()[1:-1]
-					if section in (u'massstoragedrivers', u'oembootfiles'):
+			with codecs.open(sifFile, 'r', 'cp1250') as sif:
+				for line in sif.readlines():
+					if line.strip():
+						logger.debug2(u"Current sif file content: %s" % line.rstrip())
+					if line.strip().startswith(u'['):
+						section = line.strip().lower()[1:-1]
+						if section in (u'massstoragedrivers', u'oembootfiles'):
+							continue
+					if section == u'massstoragedrivers':
+						massStorageDriverLines.append(line)
 						continue
-				if section == u'massstoragedrivers':
-					massStorageDriverLines.append(line)
-					continue
-				if section == u'oembootfiles':
-					oemBootFileLines.append(line)
-					continue
-				lines.append(line)
-			sif.close()
+					if section == u'oembootfiles':
+						oemBootFileLines.append(line)
+						continue
+					lines.append(line)
 
 			logger.info(u"Patching sections for driver '%s'" % description)
 
@@ -413,28 +412,28 @@ def integrateWindowsTextmodeDrivers(driverDirectory, destination, devices, sifFi
 			logger.debug(oemBootFileLines)
 			lines.extend(oemBootFileLines)
 
-			sif = codecs.open(sifFile, 'w', 'cp1250')
-			sif.writelines(lines)
-			sif.close()
+			with codecs.open(sifFile, 'w', 'cp1250') as sif:
+				sif.writelines(lines)
 
 
 def integrateAdditionalWindowsDrivers(driverSourceDirectory, driverDestinationDirectory, additionalDrivers, messageSubject=None, srcRepository=None, auditHardwareOnHosts=None):
 	driverSourceDirectory = forceFilename(driverSourceDirectory)
 	driverDestinationDirectory = forceFilename(driverDestinationDirectory)
-	if type(additionalDrivers) is not list:
+	if not isinstance(additionalDrivers, list):
 		additionalDrivers = [additionalDriver.strip() for additionalDriver in forceUnicodeList(additionalDrivers.split(','))]
 	else:
 		additionalDrivers = forceUnicodeList(additionalDrivers)
 
-	exists  = os.path.exists
+	if not auditHardwareOnHosts:
+		auditHardwareOnHosts = []
+
+	exists = os.path.exists
 	listdir = os.listdir
-	copy    = System.copy
 	if srcRepository:
 		if not isinstance(srcRepository, Repository):
 			raise Exception(u"Not a repository: %s" % srcRepository)
-		exists  = srcRepository.exists
+		exists = srcRepository.exists
 		listdir = srcRepository.listdir
-		copy    = srcRepository.copy
 
 	logger.info(u"Adding additional drivers")
 
@@ -455,48 +454,62 @@ def integrateAdditionalWindowsDrivers(driverSourceDirectory, driverDestinationDi
 	if exists(rulesdir) and "COMPUTER_SYSTEM" in auditInfoByClass:
 		logger.info(u"Checking if automated integrating of additional drivers are possible")
 		auditHardwareOnHost = auditInfoByClass["COMPUTER_SYSTEM"]
-		vendorFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.vendor)
-		modelFromHost  = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.model)
+		vendorFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.vendor or "")
+		modelFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.model or "")
+		skuFromHost = auditHardwareOnHost.sku or ""
+		skuLabel = ""
+		fallbackPath = ""
 
 		if vendorFromHost and modelFromHost:
 			vendordirectories = listdir(rulesdir)
-			if not vendorFromHost in vendordirectories:
+			if vendorFromHost not in vendordirectories:
 				if vendorFromHost.endswith(".") or vendorFromHost.endswith(" "):
 					vendorFromHost = "%s_" % vendorFromHost[:-1]
 
 			for vendordirectory in vendordirectories:
 				if vendordirectory.lower() == vendorFromHost.lower():
-					modeldirectories = listdir(os.path.join(rulesdir,vendordirectory))
-					if not modelFromHost in modeldirectories:
+					modeldirectories = listdir(os.path.join(rulesdir, vendordirectory))
+					if skuFromHost and skuFromHost in modelFromHost:
+						skuLabel = "(%s)" % skuFromHost
+					if modelFromHost not in modeldirectories:
 						if modelFromHost.endswith(".") or modelFromHost.endswith(" "):
 							modelFromHost = "%s_" % modelFromHost[:-1]
 					for modeldirectory in modeldirectories:
 						if modeldirectory.lower() == modelFromHost.lower():
-							additionalDrivers.append(os.path.join("byAudit" , vendordirectory, modeldirectory))
+							logger.info("ByAudit: Exact match found.")
+							additionalDrivers.append(os.path.join("byAudit", vendordirectory, modeldirectory))
 							byAuditIntegrated = True
+							break
+						elif modeldirectory.lower() == modelFromHost.replace(skuLabel, "").strip().lower():
+							fallbackPath = os.path.join("byAudit", vendordirectory, modeldirectory)
+					if not byAuditIntegrated and fallbackPath:
+						logger.info("ByAudit: No Exact match found but model without sku found. Using Directory: '%s'" % modeldirectory )
+						additionalDrivers.append(fallbackPath)
+						byAuditIntegrated = True
+					break
 
-	if not byAuditIntegrated and exists(rulesdir) and auditInfoByClass.has_key("BASE_BOARD"):
+	if not byAuditIntegrated and exists(rulesdir) and "BASE_BOARD" in auditInfoByClass:
 		logger.info(u"Checking if mainboard-fallback for automated integrating of additional drivers are possible")
 		auditHardwareOnHost = auditInfoByClass["BASE_BOARD"]
-		vendorFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.vendor)
-		productFromHost  = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.product)
+		vendorFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.vendor or "")
+		productFromHost = re.sub("[\<\>\?\"\:\|\\\/\*]", "_", auditHardwareOnHost.product or "")
 
 		if vendorFromHost and productFromHost:
 			vendordirectories = listdir(rulesdir)
-			if not vendorFromHost in vendordirectories:
+			if vendorFromHost not in vendordirectories:
 				if vendorFromHost.endswith(".") or vendorFromHost.endswith(" "):
 					vendorFromHost = "%s_" % vendorFromHost[:-1]
 
 			for vendordirectory in vendordirectories:
 				if vendordirectory.lower() == vendorFromHost.lower():
-					productdirectories = listdir(os.path.join(rulesdir,vendordirectory))
-					if not productFromHost in productdirectories:
+					productdirectories = listdir(os.path.join(rulesdir, vendordirectory))
+					if productFromHost not in productdirectories:
 						if productFromHost.endswith(".") or productFromHost.endswith(" "):
 							productFromHost = "%s_" % productFromHost[:-1]
 
 					for productdirectory in productdirectories:
 						if productdirectory.lower() == productFromHost.lower():
-							additionalDrivers.append(os.path.join("byAudit" , vendordirectory, productdirectory))
+							additionalDrivers.append(os.path.join("byAudit", vendordirectory, productdirectory))
 
 	driverDirectories = []
 	for additionalDriver in additionalDrivers:
@@ -529,8 +542,9 @@ def integrateAdditionalWindowsDrivers(driverSourceDirectory, driverDestinationDi
 					if entry.lower() == 'txtsetup.oem':
 						additionalDriverDir = parentDir
 						break
-			except Exception as e:
-				logger.debug(e)
+			except Exception as error:
+				logger.debug(error)
+
 			if additionalDriverDir in driverDirectories:
 				continue
 			logger.info(u"Adding additional driver dir '%s'" % additionalDriverDir)
